@@ -5,25 +5,28 @@ use std::convert::TryInto;
 #[cfg(not(feature = "library"))]
 use cosmwasm_std::entry_point;
 use cosmwasm_std::{
-    to_binary, Addr, Binary, BlockInfo, Decimal, Deps, DepsMut, Env, MessageInfo, Order, Reply,
-    StdError, StdResult, Timestamp, WasmMsg,
+    to_binary, Addr, Binary, BlockInfo, CustomQuery, Decimal, Deps, DepsMut, Env, MessageInfo,
+    Order, QueryRequest, Reply, StdError, StdResult, Timestamp, WasmMsg,
 };
 
-use cw2::set_contract_version;
+use cw2::{get_contract_version, set_contract_version};
 use cw_controllers::AdminError;
 use cw_storage_plus::Bound;
 use cw_utils::{maybe_addr, parse_reply_instantiate_data};
 
+use semver::Version;
 use tg4::{Member, Tg4Contract};
 use tg_bindings::{
     request_privileges, Ed25519Pubkey, Evidence, EvidenceType, Privilege, PrivilegeChangeMsg,
-    Pubkey, TgradeMsg, TgradeSudoMsg, ValidatorDiff, ValidatorUpdate,
+    Pubkey, TgradeMsg, TgradeQuery, TgradeSudoMsg, ToAddress, ValidatorDiff, ValidatorUpdate,
+    ValidatorVoteResponse,
 };
 use tg_utils::{ensure_from_older_version, JailingDuration, SlashMsg, ADMIN};
 
 use crate::error::ContractError;
+use crate::migration::migrate_jailing_period;
 use crate::msg::{
-    EpochResponse, ExecuteMsg, InstantiateMsg, InstantiateResponse, JailingPeriod,
+    EpochResponse, ExecuteMsg, InstantiateMsg, InstantiateResponse, JailingEnd, JailingPeriod,
     ListActiveValidatorsResponse, ListValidatorResponse, ListValidatorSlashingResponse, MigrateMsg,
     OperatorResponse, QueryMsg, RewardsDistribution, RewardsInstantiateMsg, ValidatorMetadata,
     ValidatorResponse,
@@ -31,7 +34,7 @@ use crate::msg::{
 use crate::rewards::pay_block_rewards;
 use crate::state::{
     operators, Config, EpochInfo, OperatorInfo, ValidatorInfo, ValidatorSlashing, CONFIG, EPOCH,
-    JAIL, VALIDATORS, VALIDATOR_SLASHING, VALIDATOR_START_HEIGHT,
+    JAIL, PENDING_VALIDATORS, VALIDATORS, VALIDATOR_SLASHING, VALIDATOR_START_HEIGHT,
 };
 
 // version info for migration info
@@ -46,7 +49,7 @@ pub type SubMsg = cosmwasm_std::SubMsg<TgradeMsg>;
 
 #[cfg_attr(not(feature = "library"), entry_point)]
 pub fn instantiate(
-    deps: DepsMut,
+    deps: DepsMut<TgradeQuery>,
     env: Env,
     _info: MessageInfo,
     msg: InstantiateMsg,
@@ -75,6 +78,8 @@ pub fn instantiate(
         distribution_contracts,
         // Will be overwritten in reply for rewards contract instantiation
         validator_group: Addr::unchecked(""),
+        verify_validators: msg.verify_validators,
+        offline_jail_duration: msg.offline_jail_duration,
     };
     CONFIG.save(deps.storage, &cfg)?;
 
@@ -140,7 +145,7 @@ pub fn instantiate(
 
 #[cfg_attr(not(feature = "library"), entry_point)]
 pub fn execute(
-    deps: DepsMut,
+    deps: DepsMut<TgradeQuery>,
     env: Env,
     info: MessageInfo,
     msg: ExecuteMsg,
@@ -174,8 +179,8 @@ pub fn execute(
     }
 }
 
-fn execute_update_config(
-    deps: DepsMut,
+fn execute_update_config<Q: CustomQuery>(
+    deps: DepsMut<Q>,
     info: MessageInfo,
     min_points: Option<u64>,
     max_validators: Option<u32>,
@@ -199,8 +204,8 @@ fn execute_update_config(
     Ok(res)
 }
 
-fn execute_register_validator_key(
-    deps: DepsMut,
+fn execute_register_validator_key<Q: CustomQuery>(
+    deps: DepsMut<Q>,
     _env: Env,
     info: MessageInfo,
     pubkey: Pubkey,
@@ -231,8 +236,8 @@ fn execute_register_validator_key(
     Ok(res)
 }
 
-fn execute_update_metadata(
-    deps: DepsMut,
+fn execute_update_metadata<Q: CustomQuery>(
+    deps: DepsMut<Q>,
     _env: Env,
     info: MessageInfo,
     metadata: ValidatorMetadata,
@@ -257,8 +262,8 @@ fn execute_update_metadata(
     Ok(res)
 }
 
-fn execute_jail(
-    deps: DepsMut,
+fn execute_jail<Q: CustomQuery>(
+    deps: DepsMut<Q>,
     env: Env,
     info: MessageInfo,
     operator: String,
@@ -274,9 +279,9 @@ fn execute_jail(
         &expiration,
     )?;
 
-    let until_attr = match expiration {
-        JailingPeriod::Until(expires) => Timestamp::from(expires).to_string(),
-        JailingPeriod::Forever {} => "forever".to_owned(),
+    let until_attr = match expiration.end {
+        JailingEnd::Until(expires) => Timestamp::from(expires).to_string(),
+        JailingEnd::Forever {} => "forever".to_owned(),
     };
 
     let res = Response::new()
@@ -287,8 +292,8 @@ fn execute_jail(
     Ok(res)
 }
 
-fn execute_unjail(
-    deps: DepsMut,
+fn execute_unjail<Q: CustomQuery>(
+    deps: DepsMut<Q>,
     env: Env,
     info: MessageInfo,
     operator: Option<String>,
@@ -303,19 +308,17 @@ fn execute_unjail(
         return Err(AdminError::NotAdmin {}.into());
     }
 
-    match JAIL.may_load(deps.storage, operator) {
-        Err(err) => return Err(err.into()),
-        // Operator is not jailed, unjailing does nothing and succeeds
-        Ok(None) => (),
-        Ok(Some(JailingPeriod::Forever {})) => {
+    // if this is `None`, the validator was not unjailed and unjailing succeeds
+    if let Some(expiration) = JAIL.may_load(deps.storage, operator)? {
+        if expiration.is_forever() {
             return Err(ContractError::UnjailFromJailForeverForbidden {});
         }
-        // Jailing period expired or called by admin - can unjail
-        Ok(Some(expiration)) if (expiration.is_expired(&env.block) || is_admin) => {
+
+        if expiration.is_expired(&env.block) || is_admin {
             JAIL.remove(deps.storage, operator);
+        } else {
+            return Err(ContractError::JailDidNotExpire {});
         }
-        // Jail not expired
-        _ => return Err(ContractError::JailDidNotExpire {}),
     }
 
     let res = Response::new()
@@ -325,8 +328,8 @@ fn execute_unjail(
     Ok(res)
 }
 
-fn store_slashing_event(
-    deps: DepsMut,
+fn store_slashing_event<Q: CustomQuery>(
+    deps: DepsMut<Q>,
     env: &Env,
     addr: Addr,
     portion: Decimal,
@@ -342,8 +345,8 @@ fn store_slashing_event(
     Ok(())
 }
 
-fn execute_slash(
-    mut deps: DepsMut,
+fn execute_slash<Q: CustomQuery>(
+    mut deps: DepsMut<Q>,
     env: Env,
     info: MessageInfo,
     operator: String,
@@ -375,8 +378,8 @@ fn execute_slash(
 }
 
 #[cfg(debug_assertions)]
-fn execute_simulate_validators(
-    deps: DepsMut,
+fn execute_simulate_validators<Q: CustomQuery>(
+    deps: DepsMut<Q>,
     _info: MessageInfo,
     validators: Vec<ValidatorInfo>,
 ) -> Result<Response, ContractError> {
@@ -398,7 +401,7 @@ fn execute_simulate_validators(
 }
 
 #[cfg_attr(not(feature = "library"), entry_point)]
-pub fn query(deps: Deps, env: Env, msg: QueryMsg) -> Result<Binary, ContractError> {
+pub fn query(deps: Deps<TgradeQuery>, env: Env, msg: QueryMsg) -> Result<Binary, ContractError> {
     use QueryMsg::*;
     match msg {
         Configuration {} => Ok(to_binary(&CONFIG.load(deps.storage)?)?),
@@ -429,7 +432,7 @@ pub fn query(deps: Deps, env: Env, msg: QueryMsg) -> Result<Binary, ContractErro
     }
 }
 
-fn query_epoch(deps: Deps, env: Env) -> Result<EpochResponse, ContractError> {
+fn query_epoch<Q: CustomQuery>(deps: Deps<Q>, env: Env) -> Result<EpochResponse, ContractError> {
     let epoch = EPOCH.load(deps.storage)?;
     let mut next_update_time =
         Timestamp::from_seconds((epoch.current_epoch + 1) * epoch.epoch_length);
@@ -447,8 +450,8 @@ fn query_epoch(deps: Deps, env: Env) -> Result<EpochResponse, ContractError> {
     Ok(resp)
 }
 
-fn query_validator_key(
-    deps: Deps,
+fn query_validator_key<Q: CustomQuery>(
+    deps: Deps<Q>,
     env: Env,
     operator: String,
 ) -> Result<ValidatorResponse, ContractError> {
@@ -470,8 +473,8 @@ fn query_validator_key(
 const MAX_LIMIT: u32 = 100;
 const DEFAULT_LIMIT: u32 = 30;
 
-fn list_validator_keys(
-    deps: Deps,
+fn list_validator_keys<Q: CustomQuery>(
+    deps: Deps<Q>,
     env: Env,
     start_after: Option<String>,
     limit: Option<u32>,
@@ -479,7 +482,7 @@ fn list_validator_keys(
     let cfg = CONFIG.load(deps.storage)?;
     let limit = limit.unwrap_or(DEFAULT_LIMIT).min(MAX_LIMIT) as usize;
     let start_after = maybe_addr(deps.api, start_after)?;
-    let start = start_after.map(|addr| Bound::exclusive(addr.as_str()));
+    let start = start_after.as_ref().map(Bound::exclusive);
 
     let operators: StdResult<Vec<_>> = operators()
         .range(deps.storage, start, None, Order::Ascending)
@@ -506,8 +509,8 @@ fn list_validator_keys(
     })
 }
 
-fn list_active_validators(
-    deps: Deps,
+fn list_active_validators<Q: CustomQuery>(
+    deps: Deps<Q>,
     start_after: Option<String>,
     limit: Option<u32>,
 ) -> Result<ListActiveValidatorsResponse, ContractError> {
@@ -532,8 +535,8 @@ fn list_active_validators(
     })
 }
 
-fn list_jailed_validators(
-    deps: Deps,
+fn list_jailed_validators<Q: CustomQuery>(
+    deps: Deps<Q>,
     env: Env,
     start_after: Option<String>,
     limit: Option<u32>,
@@ -541,7 +544,7 @@ fn list_jailed_validators(
     let cfg = CONFIG.load(deps.storage)?;
     let limit = limit.unwrap_or(DEFAULT_LIMIT).min(MAX_LIMIT) as usize;
     let start_after = maybe_addr(deps.api, start_after)?;
-    let start = start_after.map(|addr| Bound::exclusive(addr.as_str()));
+    let start = start_after.as_ref().map(Bound::exclusive);
 
     let validators = JAIL
         .range(deps.storage, start, None, Order::Ascending)
@@ -572,16 +575,16 @@ fn list_jailed_validators(
     Ok(ListValidatorResponse { validators })
 }
 
-fn simulate_active_validators(
-    deps: Deps,
+fn simulate_active_validators<Q: CustomQuery>(
+    deps: Deps<Q>,
     env: Env,
 ) -> Result<ListActiveValidatorsResponse, ContractError> {
     let (validators, _) = calculate_validators(deps, &env)?;
     Ok(ListActiveValidatorsResponse { validators })
 }
 
-fn list_validator_slashing(
-    deps: Deps,
+fn list_validator_slashing<Q: CustomQuery>(
+    deps: Deps<Q>,
     _env: Env,
     operator: String,
 ) -> Result<ListValidatorSlashingResponse, ContractError> {
@@ -593,9 +596,9 @@ fn list_validator_slashing(
     let slashing = VALIDATOR_SLASHING
         .may_load(deps.storage, &addr)?
         .unwrap_or_default();
-    let (jailed_until, tombstoned) = match JAIL.may_load(deps.storage, &addr)? {
-        Some(JailingPeriod::Forever {}) => (None, true),
-        Some(JailingPeriod::Until(u)) => (Some(u), false),
+    let (jailed_until, tombstoned) = match JAIL.may_load(deps.storage, &addr)?.map(|j| j.end) {
+        Some(JailingEnd::Forever {}) => (None, true),
+        Some(JailingEnd::Until(u)) => (Some(u), false),
         None => (None, false),
     };
     Ok(ListValidatorSlashingResponse {
@@ -608,7 +611,11 @@ fn list_validator_slashing(
 }
 
 #[cfg_attr(not(feature = "library"), entry_point)]
-pub fn sudo(deps: DepsMut, env: Env, msg: TgradeSudoMsg) -> Result<Response, ContractError> {
+pub fn sudo(
+    deps: DepsMut<TgradeQuery>,
+    env: Env,
+    msg: TgradeSudoMsg,
+) -> Result<Response, ContractError> {
     match msg {
         TgradeSudoMsg::PrivilegeChange(change) => Ok(privilege_change(deps, change)),
         TgradeSudoMsg::EndWithValidatorUpdate {} => end_block(deps, env),
@@ -617,7 +624,7 @@ pub fn sudo(deps: DepsMut, env: Env, msg: TgradeSudoMsg) -> Result<Response, Con
     }
 }
 
-fn privilege_change(_deps: DepsMut, change: PrivilegeChangeMsg) -> Response {
+fn privilege_change<Q: CustomQuery>(_deps: DepsMut<Q>, change: PrivilegeChangeMsg) -> Response {
     match change {
         PrivilegeChangeMsg::Promoted {} => {
             let msgs = request_privileges(&[
@@ -642,7 +649,7 @@ fn is_genesis_block(block: &BlockInfo) -> bool {
     block.height < 2
 }
 
-fn end_block(deps: DepsMut, env: Env) -> Result<Response, ContractError> {
+fn end_block(deps: DepsMut<TgradeQuery>, env: Env) -> Result<Response, ContractError> {
     let cfg = CONFIG.load(deps.storage)?;
 
     // check if needed and quit early if we didn't hit epoch boundary
@@ -663,8 +670,31 @@ fn end_block(deps: DepsMut, env: Env) -> Result<Response, ContractError> {
     epoch.current_epoch = cur_epoch;
     EPOCH.save(deps.storage, &epoch)?;
 
+    if cfg.verify_validators {
+        let votes = deps
+            .querier
+            .query::<ValidatorVoteResponse>(&QueryRequest::Custom(TgradeQuery::ValidatorVotes {}))?
+            .votes;
+
+        if let Some(pending) = PENDING_VALIDATORS.may_load(deps.storage)? {
+            let expiration = JailingPeriod::from_duration(
+                JailingDuration::Duration(cfg.offline_jail_duration),
+                &env.block,
+            );
+
+            for val in pending {
+                let vote = votes
+                    .iter()
+                    .find(|vote| vote.address.as_slice() == val.1.to_address());
+                if vote.is_none() || !vote.unwrap().voted {
+                    JAIL.save(deps.storage, &val.0, &expiration)?;
+                }
+            }
+        }
+    }
+
     // calculate and store new validator set
-    let (validators, auto_unjail) = calculate_validators(deps.as_ref(), &env)?;
+    let (mut validators, auto_unjail) = calculate_validators(deps.as_ref(), &env)?;
 
     // auto unjailing
     for addr in &auto_unjail {
@@ -673,14 +703,41 @@ fn end_block(deps: DepsMut, env: Env) -> Result<Response, ContractError> {
 
     let old_validators = VALIDATORS.load(deps.storage)?;
 
-    VALIDATORS.save(deps.storage, &validators)?;
     // determine the diff to send back to tendermint
-    let (diff, add, remove) = calculate_diff(validators, old_validators);
+    let (diff, add, remove) = calculate_diff(validators.clone(), old_validators);
     let update_members = RewardsDistribution::UpdateMembers {
         add: add.clone(),
         remove: remove.clone(),
     };
 
+    if cfg.verify_validators {
+        // pending validators are validators who have just been added and have yet to verify they're online
+        // and signing blocks (or at least signing the first one)
+        let pending: Vec<_> = add
+            .iter()
+            .filter_map(|m| {
+                let addr = Addr::unchecked(&m.addr);
+                if let Ok(op) = operators().load(deps.storage, &addr) {
+                    if !op.active_validator {
+                        Some((addr, op.pubkey))
+                    } else {
+                        None
+                    }
+                } else {
+                    None
+                }
+            })
+            .collect();
+        PENDING_VALIDATORS.save(deps.storage, &pending)?;
+        for v in &mut validators {
+            if pending.iter().any(|(addr, _)| *addr == v.operator) {
+                v.power = cfg.min_points;
+            }
+        }
+    }
+
+    VALIDATORS.save(deps.storage, &validators)?;
+
     // update operators list with info about whether or not they're active validators
     for op in add {
         operators().update::<_, StdError>(deps.storage, &Addr::unchecked(op.addr), |op| {
@@ -736,8 +793,8 @@ const QUERY_LIMIT: Option<u32> = Some(30);
 
 /// Selects validators to be used for incoming epoch. Returns vector of validators info paired
 /// with vector of addresses to be un-jailed (always empty if auto un-jailing is disabled).
-fn calculate_validators(
-    deps: Deps,
+fn calculate_validators<Q: CustomQuery>(
+    deps: Deps<Q>,
     env: &Env,
 ) -> Result<(Vec<ValidatorInfo>, Vec<Addr>), ContractError> {
     let cfg = CONFIG.load(deps.storage)?;
@@ -873,7 +930,11 @@ fn calculate_diff(
 }
 
 #[cfg_attr(not(feature = "library"), entry_point)]
-pub fn migrate(deps: DepsMut, _env: Env, msg: MigrateMsg) -> Result<Response, ContractError> {
+pub fn migrate(
+    mut deps: DepsMut<TgradeQuery>,
+    _env: Env,
+    msg: MigrateMsg,
+) -> Result<Response, ContractError> {
     ensure_from_older_version(deps.storage, CONTRACT_NAME, CONTRACT_VERSION)?;
 
     CONFIG.update::<_, StdError>(deps.storage, |mut cfg| {
@@ -886,6 +947,12 @@ pub fn migrate(deps: DepsMut, _env: Env, msg: MigrateMsg) -> Result<Response, Co
         Ok(cfg)
     })?;
 
+    let stored_version = get_contract_version(deps.storage)?;
+    // Unwrapping as version check before would fail if stored version is invalid
+    let stored_version: Version = stored_version.version.parse().unwrap();
+
+    migrate_jailing_period(deps.branch(), &stored_version)?;
+
     Ok(Response::new())
 }
 
@@ -899,8 +966,8 @@ mod evidence {
     /// suspect, this function computes sha256 hashes for all existing validator and
     /// compares result with suspect. It is acceptable approach, since it shouldn't
     /// happen too often.
-    pub fn find_matching_validator(
-        deps: Deps,
+    pub fn find_matching_validator<Q: CustomQuery>(
+        deps: Deps<Q>,
         suspect: &Validator,
         evidence_height: u64,
     ) -> Result<Option<Addr>, cosmwasm_std::StdError> {
@@ -942,7 +1009,7 @@ mod evidence {
 /// If some validators are caught on malicious behavior (for example double signing),
 /// they are reported and punished on begin of next block.
 fn begin_block(
-    mut deps: DepsMut,
+    mut deps: DepsMut<TgradeQuery>,
     env: Env,
     evidences: Vec<Evidence>,
 ) -> Result<Response, ContractError> {
@@ -975,7 +1042,11 @@ fn begin_block(
                     config.double_sign_slash_ratio,
                 )?;
 
-                JAIL.save(deps.storage, &validator, &JailingPeriod::Forever {})?;
+                JAIL.save(
+                    deps.storage,
+                    &validator,
+                    &JailingPeriod::from_duration(JailingDuration::Forever {}, &env.block),
+                )?;
 
                 response = response
                     .clone()
@@ -991,15 +1062,15 @@ fn begin_block(
 }
 
 #[cfg_attr(not(feature = "library"), entry_point)]
-pub fn reply(deps: DepsMut, env: Env, msg: Reply) -> Result<Response, ContractError> {
+pub fn reply(deps: DepsMut<TgradeQuery>, env: Env, msg: Reply) -> Result<Response, ContractError> {
     match msg.id {
         REWARDS_INIT_REPLY_ID => rewards_instantiate_reply(deps, env, msg),
         _ => Err(ContractError::UnrecognisedReply(msg.id)),
     }
 }
 
-pub fn rewards_instantiate_reply(
-    deps: DepsMut,
+pub fn rewards_instantiate_reply<Q: CustomQuery>(
+    deps: DepsMut<Q>,
     _env: Env,
     msg: Reply,
 ) -> Result<Response, ContractError> {
