@@ -5,11 +5,11 @@ use std::ops::Add;
 
 use tg4::Member;
 use tg_bindings::{Ed25519Pubkey, Pubkey};
-use tg_utils::{Expiration, JailingDuration};
+use tg_utils::{Duration, Expiration, JailingDuration};
 
 use crate::error::ContractError;
 use crate::state::{DistributionContract, OperatorInfo, ValidatorInfo, ValidatorSlashing};
-use cosmwasm_std::{Addr, Api, BlockInfo, Coin, Decimal};
+use cosmwasm_std::{Addr, Api, BlockInfo, Coin, Decimal, Timestamp};
 
 #[derive(Serialize, Deserialize, Clone, PartialEq, JsonSchema, Debug)]
 pub struct InstantiateMsg {
@@ -90,6 +90,17 @@ pub struct InstantiateMsg {
     ///
     /// This contract has to support all the `RewardsDistribution` messages
     pub validator_group_code_id: u64,
+
+    /// When a validator joins the valset, verify they sign the first block since joining
+    /// or jail them for a period otherwise.
+    ///
+    /// The verification happens every time the validator becomes an active validator,
+    /// including when they are unjailed or when they just gain enough power to participate.
+    pub verify_validators: bool,
+
+    /// The duration to jail a validator for in case they don't sign their first epoch
+    /// boundary block. After the period, they have to pass verification again, ad infinitum.
+    pub offline_jail_duration: Duration,
 }
 
 impl InstantiateMsg {
@@ -395,25 +406,38 @@ impl OperatorResponse {
     }
 }
 
+#[derive(Serialize, Deserialize, Clone, PartialEq, JsonSchema, Debug)]
+pub struct JailingPeriod {
+    pub start: Timestamp,
+    pub end: JailingEnd,
+}
+
 #[derive(Serialize, Deserialize, Clone, PartialEq, JsonSchema, Debug)]
 #[serde(rename_all = "snake_case")]
-pub enum JailingPeriod {
+pub enum JailingEnd {
     Until(Expiration),
     Forever {},
 }
 
 impl JailingPeriod {
     pub fn from_duration(duration: JailingDuration, block: &BlockInfo) -> Self {
-        match duration {
-            JailingDuration::Duration(duration) => Self::Until(duration.after(block)),
-            JailingDuration::Forever {} => Self::Forever {},
+        Self {
+            start: block.time,
+            end: match duration {
+                JailingDuration::Duration(duration) => JailingEnd::Until(duration.after(block)),
+                JailingDuration::Forever {} => JailingEnd::Forever {},
+            },
         }
     }
 
+    pub fn is_forever(&self) -> bool {
+        matches!(self.end, JailingEnd::Forever {})
+    }
+
     pub fn is_expired(&self, block: &BlockInfo) -> bool {
-        match self {
-            Self::Forever {} => false,
-            Self::Until(expires) => expires.is_expired(block),
+        match self.end {
+            JailingEnd::Forever {} => false,
+            JailingEnd::Until(expires) => expires.is_expired(block),
         }
     }
 }
@@ -519,6 +543,8 @@ mod test {
             double_sign_slash_ratio: Decimal::percent(50),
             distribution_contracts: UnvalidatedDistributionContracts::default(),
             validator_group_code_id: 0,
+            verify_validators: false,
+            offline_jail_duration: Duration::new(0),
         };
         proper.validate().unwrap();
 
