@@ -1,8 +1,9 @@
 use crate::error::ContractError;
-use crate::msg::JailingPeriod;
+use crate::msg::{JailingEnd, ValidatorResponse};
 
 use super::helpers::{assert_active_validators, assert_operators, members_init};
 use super::suite::SuiteBuilder;
+use cosmwasm_std::{StdResult, Timestamp};
 use cw_controllers::AdminError;
 use tg_utils::{Duration, Expiration, JailingDuration};
 
@@ -37,7 +38,7 @@ fn only_admin_can_jail() {
         ))
     );
 
-    let jailed_until = JailingPeriod::Until(Duration::new(3600).after(&suite.app().block_info()));
+    let jailed_until = JailingEnd::Until(Duration::new(3600).after(&suite.app().block_info()));
 
     // Non-admin cannot jail forever
     let err = suite
@@ -67,7 +68,7 @@ fn only_admin_can_jail() {
         &suite.list_validators(None, None).unwrap(),
         &[
             (members[0], None),
-            (members[1], Some(JailingPeriod::Forever {})),
+            (members[1], Some(JailingEnd::Forever {})),
             (members[2], Some(jailed_until)),
             (members[3], None),
         ],
@@ -109,7 +110,7 @@ fn admin_can_unjail_almost_anyone() {
         &suite.list_validators(None, None).unwrap(),
         &[
             (members[0], None),
-            (members[1], Some(JailingPeriod::Forever {})),
+            (members[1], Some(JailingEnd::Forever {})),
             (members[2], None),
             (members[3], None),
         ],
@@ -130,7 +131,7 @@ fn anyone_can_unjail_self_after_period() {
     suite.jail(&admin, members[1], Duration::new(3600)).unwrap();
     suite.jail(&admin, members[2], Duration::new(3600)).unwrap();
 
-    let jailed_until = JailingPeriod::Until(Duration::new(3600).after(&suite.app().block_info()));
+    let jailed_until = JailingEnd::Until(Duration::new(3600).after(&suite.app().block_info()));
 
     // Move a little bit forward, so some time passed, but not eough for any jailing to
     // expire
@@ -240,7 +241,7 @@ fn auto_unjail() {
 
     let admin = suite.admin().to_owned();
 
-    let jailed_until = JailingPeriod::Until(Duration::new(3600).after(&suite.app().block_info()));
+    let jailed_until = JailingEnd::Until(Duration::new(3600).after(&suite.app().block_info()));
 
     // Jailing some operators to begin with
     suite.jail(&admin, members[0], Duration::new(3600)).unwrap();
@@ -256,7 +257,7 @@ fn auto_unjail() {
         &suite.list_validators(None, None).unwrap(),
         &[
             (members[0], Some(jailed_until)),
-            (members[1], Some(JailingPeriod::Forever {})),
+            (members[1], Some(JailingEnd::Forever {})),
             (members[2], None),
             (members[3], None),
         ],
@@ -276,7 +277,7 @@ fn auto_unjail() {
         &suite.list_validators(None, None).unwrap(),
         &[
             (members[0], None),
-            (members[1], Some(JailingPeriod::Forever {})),
+            (members[1], Some(JailingEnd::Forever {})),
             (members[2], None),
             (members[3], None),
         ],
@@ -348,3 +349,90 @@ fn jailed_validators_show_up_as_inactive_when_listed() {
     assert!(operators[2].active_validator);
     assert!(operators[3].active_validator);
 }
+
+#[test]
+fn list_jailed_validators_all() {
+    let members = vec!["member1", "member2", "member3", "member4"];
+    let mut suite = SuiteBuilder::new()
+        .with_engagement(&members_init(&members, &[2, 3, 5, 8]))
+        .with_operators(&members)
+        .build();
+    let admin = suite.admin().to_owned();
+
+    // Jailing operators as test prerequirements
+    suite.jail(&admin, members[0], Duration::new(3600)).unwrap();
+    suite.jail(&admin, members[2], Duration::new(7200)).unwrap();
+
+    let operators = suite.list_jailed_validators(None, None).unwrap();
+    assert_eq!(operators.len(), 2);
+    assert_eq!(operators[0].operator, members[0]);
+    assert_eq!(operators[1].operator, members[2]);
+
+    // Moving forward so jailing periods expired
+    suite.advance_seconds(4000).unwrap();
+
+    // Unjail one of the jailed ops
+    suite.unjail(&admin, members[0]).unwrap();
+
+    let operators = suite.list_jailed_validators(None, None).unwrap();
+    assert_eq!(operators.len(), 1);
+    assert_eq!(operators[0].operator, members[2]);
+}
+
+#[test]
+fn list_jailed_validators_with_pagination() {
+    let members = vec!["member1", "member2", "member3", "member4", "member5"];
+    let mut suite = SuiteBuilder::new()
+        .with_engagement(&members_init(&members, &[2, 3, 5, 8, 10]))
+        .with_operators(&members)
+        .build();
+    let admin = suite.admin().to_owned();
+
+    // Jailing operators as test prerequirements
+    suite.jail(&admin, members[0], Duration::new(3600)).unwrap();
+    suite.jail(&admin, members[1], Duration::new(3600)).unwrap();
+    suite.jail(&admin, members[2], Duration::new(7200)).unwrap();
+    suite.jail(&admin, members[3], Duration::new(7200)).unwrap();
+    suite.jail(&admin, members[4], Duration::new(7200)).unwrap();
+
+    let operators = suite.list_jailed_validators(None, None).unwrap();
+    assert_eq!(operators.len(), 5);
+
+    let operators = suite.list_jailed_validators(None, 3).unwrap();
+    assert_eq!(operators.len(), 3);
+    assert_eq!(operators[0].operator, members[0]);
+    assert_eq!(operators[1].operator, members[1]);
+    assert_eq!(operators[2].operator, members[2]);
+
+    let operators = suite
+        .list_jailed_validators(operators.last().unwrap().operator.clone(), None)
+        .unwrap();
+    assert_eq!(operators.len(), 2);
+    assert_eq!(operators[0].operator, members[3]);
+    assert_eq!(operators[1].operator, members[4]);
+}
+
+#[test]
+fn jailing_duration_start_is_provided() {
+    let members = vec!["member1", "member2"];
+    let mut suite = SuiteBuilder::new()
+        .with_engagement(&members_init(&members, &[2, 3, 5, 8, 10]))
+        .with_operators(&members)
+        .build();
+    let admin = suite.admin().to_owned();
+
+    fn jail_start(val: StdResult<ValidatorResponse>) -> Timestamp {
+        val.unwrap().validator.unwrap().jailed_until.unwrap().start
+    }
+
+    let time1 = suite.app().block_info().time;
+    suite.jail(&admin, members[0], Duration::new(3600)).unwrap();
+    assert_eq!(time1, jail_start(suite.validator(members[0])));
+
+    suite.advance_seconds(100).unwrap();
+    let time2 = suite.app().block_info().time;
+    suite.jail(&admin, members[1], Duration::new(3600)).unwrap();
+    assert_eq!(time2, time1.plus_seconds(100));
+    assert_eq!(time1, jail_start(suite.validator(members[0])));
+    assert_eq!(time2, jail_start(suite.validator(members[1])));
+}
