@@ -1,5 +1,6 @@
 #![cfg(test)]
 use cosmwasm_std::{coin, Addr, Decimal};
+use tg_utils::Duration;
 
 use crate::multitest::suite::SuiteBuilder;
 use crate::state::{Config, ValidatorInfo};
@@ -15,7 +16,7 @@ fn init_and_query_state() {
     let suite = SuiteBuilder::new()
         .with_stake("usdc", 100u128)
         .with_operators(&ops)
-        .with_min_weight(5)
+        .with_min_points(5)
         .with_max_validators(10)
         .with_epoch_reward(epoch_reward.clone())
         .build();
@@ -26,7 +27,7 @@ fn init_and_query_state() {
         cfg,
         Config {
             membership: cfg.membership.clone(),
-            min_weight: 5,
+            min_points: 5,
             max_validators: 10,
             scaling: None,
             epoch_reward,
@@ -34,12 +35,14 @@ fn init_and_query_state() {
             auto_unjail: false,
             double_sign_slash_ratio: Decimal::percent(50),
             distribution_contracts: vec![],
-            rewards_contract: cfg.rewards_contract.clone(),
+            validator_group: cfg.validator_group.clone(),
+            verify_validators: false,
+            offline_jail_duration: Duration::new(0),
         }
     );
 
     // no initial active set
-    let active = suite.list_active_validators().unwrap();
+    let active = suite.list_active_validators(None, None).unwrap();
     assert_eq!(active, vec![]);
 
     // check a validator is set
@@ -58,8 +61,8 @@ fn init_and_query_state() {
 #[test]
 fn simulate_validators() {
     let bond_denom = "tgrade";
-    let tokens_per_weight = 100u128;
-    let min_weight = 2;
+    let tokens_per_points = 100u128;
+    let min_points = 2;
 
     let ops_owned = addrs(24);
     let operators: Vec<_> = ops_owned.iter().map(String::as_str).collect();
@@ -72,24 +75,24 @@ fn simulate_validators() {
         .collect();
 
     let mut suite = SuiteBuilder::new()
-        .with_stake(bond_denom, tokens_per_weight)
+        .with_stake(bond_denom, tokens_per_points)
         .with_operators(&operators)
         .with_funds(&operator_balances)
-        .with_min_weight(min_weight)
+        .with_min_points(min_points)
         .with_max_validators(10)
         .with_epoch_reward(coin(50_000, "usdc"))
         .build();
 
     // what do we expect?
     // 1..24 have pubkeys registered, we take the top 10, but none have stake yet, so zero
-    let active = suite.list_active_validators().unwrap();
+    let active = suite.list_active_validators(None, None).unwrap();
     assert_eq!(0, active.len());
 
-    // One member bonds needed tokens to have enough weight
+    // One member bonds needed tokens to have enough points
     let op1_addr = Addr::unchecked(operators[0]);
 
     // First, he does not bond enough tokens
-    let stake = cosmwasm_std::coins(tokens_per_weight * min_weight as u128 - 1u128, bond_denom);
+    let stake = cosmwasm_std::coins(tokens_per_points * min_points as u128 - 1u128, bond_denom);
     suite.bond(&op1_addr, &stake).unwrap();
 
     // what do we expect?
@@ -109,15 +112,14 @@ fn simulate_validators() {
     let expected: Vec<_> = vec![ValidatorInfo {
         operator: op1_addr.clone(),
         validator_pubkey: valid_operator(op1_addr.as_ref()).validator_pubkey,
-        metadata: valid_operator(op1_addr.as_ref()).metadata,
-        power: min_weight,
+        power: min_points,
     }];
     assert_eq!(expected, active);
 
     // Other member bonds twice the minimum amount
     let op2_addr = Addr::unchecked(operators[1]);
 
-    let stake = cosmwasm_std::coins(tokens_per_weight * min_weight as u128 * 2u128, bond_denom);
+    let stake = cosmwasm_std::coins(tokens_per_points * min_points as u128 * 2u128, bond_denom);
     suite.bond(&op2_addr, &stake).unwrap();
 
     // what do we expect?
@@ -130,14 +132,12 @@ fn simulate_validators() {
         ValidatorInfo {
             operator: op2_addr.clone(),
             validator_pubkey: valid_operator(op2_addr.as_ref()).validator_pubkey,
-            metadata: valid_operator(op2_addr.as_ref()).metadata,
-            power: min_weight * 2,
+            power: min_points * 2,
         },
         ValidatorInfo {
             operator: op1_addr.clone(),
             validator_pubkey: valid_operator(op1_addr.as_ref()).validator_pubkey,
-            metadata: valid_operator(op1_addr.as_ref()).metadata,
-            power: min_weight,
+            power: min_points,
         },
     ];
     assert_eq!(expected, active);
@@ -146,7 +146,7 @@ fn simulate_validators() {
     let op3_addr = Addr::unchecked(operators[2]);
 
     let stake = cosmwasm_std::coins(
-        tokens_per_weight * min_weight as u128 * 3u128 - 1u128,
+        tokens_per_points * min_points as u128 * 3u128 - 1u128,
         bond_denom,
     );
     suite.bond(&op3_addr, &stake).unwrap();
@@ -161,20 +161,17 @@ fn simulate_validators() {
         ValidatorInfo {
             operator: op3_addr.clone(),
             validator_pubkey: valid_operator(op3_addr.as_ref()).validator_pubkey,
-            metadata: valid_operator(op3_addr.as_ref()).metadata,
-            power: min_weight * 3 - 1,
+            power: min_points * 3 - 1,
         },
         ValidatorInfo {
             operator: op2_addr.clone(),
             validator_pubkey: valid_operator(op2_addr.as_ref()).validator_pubkey,
-            metadata: valid_operator(op2_addr.as_ref()).metadata,
-            power: min_weight * 2,
+            power: min_points * 2,
         },
         ValidatorInfo {
             operator: op1_addr.clone(),
             validator_pubkey: valid_operator(op1_addr.as_ref()).validator_pubkey,
-            metadata: valid_operator(op1_addr.as_ref()).metadata,
-            power: min_weight,
+            power: min_points,
         },
     ];
     assert_eq!(expected, active);
@@ -193,14 +190,12 @@ fn simulate_validators() {
         ValidatorInfo {
             operator: op3_addr.clone(),
             validator_pubkey: valid_operator(op3_addr.as_ref()).validator_pubkey,
-            metadata: valid_operator(op3_addr.as_ref()).metadata,
-            power: min_weight * 3 - 1,
+            power: min_points * 3 - 1,
         },
         ValidatorInfo {
             operator: op2_addr.clone(),
             validator_pubkey: valid_operator(op2_addr.as_ref()).validator_pubkey,
-            metadata: valid_operator(op2_addr.as_ref()).metadata,
-            power: min_weight * 2,
+            power: min_points * 2,
         },
     ];
     assert_eq!(expected, active);
