@@ -9,13 +9,13 @@ use cosmwasm_std::{
 use cw_multi_test::{next_block, AppResponse, Contract, ContractWrapper, CosmosRouter, Executor};
 use derivative::Derivative;
 use tg4::{AdminResponse, Member};
-use tg_bindings::{Evidence, Pubkey, TgradeMsg, ValidatorDiff};
+use tg_bindings::{Evidence, Pubkey, TgradeMsg, TgradeQuery, ValidatorDiff, ValidatorVote};
 use tg_bindings_test::TgradeApp;
 use tg_utils::{Duration, JailingDuration};
 
 use crate::msg::OperatorInitInfo;
 
-pub fn contract_engagement() -> Box<dyn Contract<TgradeMsg>> {
+pub fn contract_engagement() -> Box<dyn Contract<TgradeMsg, TgradeQuery>> {
     let contract = ContractWrapper::new(
         tg4_engagement::contract::execute,
         tg4_engagement::contract::instantiate,
@@ -24,7 +24,7 @@ pub fn contract_engagement() -> Box<dyn Contract<TgradeMsg>> {
     Box::new(contract)
 }
 
-pub fn contract_stake() -> Box<dyn Contract<TgradeMsg>> {
+pub fn contract_stake() -> Box<dyn Contract<TgradeMsg, TgradeQuery>> {
     let contract = ContractWrapper::new(
         tg4_stake::contract::execute,
         tg4_stake::contract::instantiate,
@@ -33,7 +33,7 @@ pub fn contract_stake() -> Box<dyn Contract<TgradeMsg>> {
     Box::new(contract)
 }
 
-pub fn contract_valset() -> Box<dyn Contract<TgradeMsg>> {
+pub fn contract_valset() -> Box<dyn Contract<TgradeMsg, TgradeQuery>> {
     let contract = ContractWrapper::new(
         crate::contract::execute,
         crate::contract::instantiate,
@@ -100,6 +100,7 @@ pub struct SuiteBuilder {
     distribution_configs: Vec<DistributionConfig>,
     /// Funds to add on init per address
     init_funds: Vec<(String, Vec<Coin>)>,
+    verify_validators: Option<Duration>,
 }
 
 impl SuiteBuilder {
@@ -164,6 +165,11 @@ impl SuiteBuilder {
         self
     }
 
+    pub fn with_verify_validators(mut self, duration: u64) -> Self {
+        self.verify_validators = Some(Duration::new(duration));
+        self
+    }
+
     pub fn with_epoch_reward(mut self, epoch_reward: Coin) -> Self {
         self.epoch_reward = epoch_reward;
         self
@@ -347,6 +353,10 @@ impl SuiteBuilder {
                         inner: distribution_contract_instantiation_info,
                     },
                     validator_group_code_id: engagement_id,
+                    verify_validators: self.verify_validators.is_some(),
+                    offline_jail_duration: self
+                        .verify_validators
+                        .unwrap_or_else(|| Duration::new(0)),
                 },
                 &[],
                 "valset",
@@ -764,7 +774,9 @@ impl Suite {
         self.app.execute_contract(
             addr.clone(),
             self.membership.clone(),
-            &tg4_stake::msg::ExecuteMsg::Bond {},
+            &tg4_stake::msg::ExecuteMsg::Bond {
+                vesting_tokens: None,
+            },
             stake,
         )
     }
@@ -790,4 +802,12 @@ impl Suite {
             self.valset_code_id,
         )
     }
+
+    pub fn set_votes(&mut self, votes: &[ValidatorVote]) -> AnyResult<()> {
+        Ok(self
+            .app
+            .init_modules(|router, _api, storage| -> StdResult<()> {
+                router.custom.set_votes(storage, votes.to_vec())
+            })?)
+    }
 }
